puts 'loading classes'

BumpSound = AS::Player.new(AS::Sound.new('ballbump.wav'))
JumpSound = AS::Player.new(AS::Sound.new('shoot.wav'))
CP::Material.set_func(0,0,CP::Material::Impulse) do|ball, wall, j|
	if ball.jump
		ball.v += cpv(0, 200)
		ball.jump = nil
		JumpSound.play
		next cpv(0,0)
	end
	
	BumpSound.play if j.length>50
	
	next j
end

FrownTex = AS::Texture.open('frown.png')

class Smile < CP::Ball
	@@tex = AS::Texture.open('smile.png')
	
	attr_accessor :jump
	
	def initialize(pos)
		super(16, 1, 1, 0.3, 0.6)
		
		self.type = 0
		self.p = pos
		
		@spr = AS::Sprite.new(cpv(16,16), @@tex)
	end
	
	def draw
		@spr.pos = p
		@spr.angle = a.rad2deg
		@spr.draw
	end
end

class Segment < CP::Segment
	def verts
		GL.Vertex(a.x, a.y)
		GL.Vertex(b.x, b.y)
	end
end

load 'levels.rb'

ArrowVerts = [
	cpv(20,0),
	cpv(40,0),
	cpv(35,5),
	cpv(35,-5),
]

ArrowLines = [
	[0,1],
	[1,2],
	[1,3],
]

def draw_arrow(p, n)
	tverts = ArrowVerts.collect{|v| v.rotate(n) + p}
	GL.Begin(GL::LINES)
		ArrowLines.each do|pair|
			a, b = tverts.values_at(*pair)
			GL.Vertex(a.x, a.y)
			GL.Vertex(b.x, b.y)
		end
	GL.End
end		

LevelFinished = AS::Player.new(AS::Sound.new('levelup.wav'))
class State < AS::Gamestate
	def initialize(level)
		super()

		@level = level
		scale = @level::Scale/80.0
		GL.LoadIdentity
		GL.Scale(scale, scale, 1)
		
		@timer = TickTimer.new(60)
		
		@space = CP::Space.new(100, 1000)
		@sml = Smile.new(@level::Start)
		@space << @sml
		
		@segs = []
		@level.get_segs.each{|seg| add_seg(seg)}
		
		@force = cpv(100,0)
		@lastDown = 0
		
		action(AS::Event.for_key(AS::Event::Key_ESCAPE)){stop(:exit)}
		action(AS::Event.for_mouse(1, true)){down}
		action(AS::Event.for_mouse(1, false)){up}
#		action(AS::Event.for_key(AS::Event::Key_BACKSPACE, false)){load 'classes.rb'}
#		action(AS::Event.for_key(AS::Event::Key_END, false)){stop(:redo)}
#		action(AS::Event.for_mouse(1)){@sml.p = cpv($mx, 480 - $my)}
	end
	
	def add_seg(seg)
		seg.type = 0
		@space << seg
		@segs << seg
	end
	
#	def input(event)
#		source, channel, id, state = event.split_event
#		if channel==0
#			if state==0
#				up
#			else
#				down
#			end
#		end
#		
#		super
#	end
	
	TAPTIME = 15
	
	def up
		delta = @timer.getTicks - @lastDown
		tap if delta < TAPTIME
		@sml.f = cpv(0,0)
	end
	
	def down
		delta = @timer.getTicks - @lastDown
		double_tap if delta < TAPTIME
		
		@lastDown = @timer.getTicks
		@sml.f = @force
	end
	
	def tap
		@force *= -1
	end
	
	def double_tap
		@sml.jump = @timer.getTicks
	end
	
	def update
		@space.update
		@sml.jump = nil if @sml.jump and @timer.getTicks - @sml.jump==15
		if @sml.p.within_dist?(@level::End, 30)
			LevelFinished.play
			stop(:next)
		end
		stop(:redo) if @sml.p.y < -20
	end
	
	def draw
		@sml.draw
		p = @level::End
		FrownTex.draw_centered(p.x, p.y, 16)
		
		AS.no_tex_mode
		draw_arrow(@sml.p, @force.normalize)
		
		GL.Begin(GL::LINES)
			@segs.send_each(:verts)
		GL.End
		
		@timer.stepTick{}
	end
end
